"""
Algebra ideal elements
"""

#*****************************************************************************
#  Copyright (C) 2005 David Kohel <kohel@maths.usyd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful, 
#    but WITHOUT ANY WARRANTY; without even the implied warranty 
#    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
#  See the GNU General Public License for more details; the full text 
#  is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.rings.polynomial_ring import PolynomialRing
from sage.algebras.algebra_element import AlgebraElement

class AlgebraIdealElement(AlgebraElement):
    """
    An element of an ideal in an algebra, stored as an element
    of the algebra in which it embedds.
    """
    def __init__(self, A, x):
        """
        Create the element x of the quaternion order A.
        """
        AlgebraElement.__init__(self, A)
        self.__algebra_element = x


class AlgebraOrderIdealElement(AlgebraIdealElement):
    """
    An element of an ideal for an order in an algebra, stored as
    an element of the order in which it embedds.
    """
    def __init__(self, A, x):
        """
        Create the element x of the quaternion order A.
        """
        AlgebraIdealElement.__init__(self, A, x)

    def __repr__(self):
        """
        """
        # Does this print as desired? 
        return str(self.__algebra_element)

    def trace(self):
        """
        The trace of the element with respect to its action
        by left or right multiplication on the order.
	"""
        R = self.parent().base_ring()
        return R(self.__algebra_element.trace())
		
    def norm(self):
        """
        The determinant (= norm) of the element with respect to
        its action by left or right multiplication on the order.
	"""
        R = self.parent().base_ring()
        return R(self.__algebra_element.norm())

    def charpoly(self):
        """
        The characteristic polynomial of the element with respect
        to its action by left or right multiplication on the order.
	"""
        R = self.parent().base_ring()
        P = PolynomialRing(R)
        return P(self.__algebra_element.charpoly())

    characteristic_polynomial = charpoly
    
    def minpoly(self):
        """
        The minimal polynomial of the element with respect to its 
        action by left or right multiplication on the order.
	"""
        R = self.parent().base_ring()
        P = PolynomialRing(R)
        return P(self.__algebra_element.minpoly())

    minimal_polynomial = minpoly
