"""
Algebra orders
"""

#*****************************************************************************
#  Copyright (C) 2005 David Kohel <kohel@maths.usyd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful, 
#    but WITHOUT ANY WARRANTY; without even the implied warranty 
#    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
#  See the GNU General Public License for more details; the full text 
#  is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.algebras.algebra import Algebra

def basis_helper(A):
    """
    This should determine a basis with first element 1, but for the moment
    we assume that the generators are themselves a basis.
    """
    return A.__gens

class AlgebraOrder(Algebra):
    """
    An order in an algebra.
    """
    def __init__(self, A, R, gens = []):
        if not isinstance(A, Algebra):
            raise TypeError, "Argument A must be an algebra."
        Algebra.__init__(self, R)
        self.__ambient_algebra = A
        self.__base_ring = R
        self.__gens = gens

    def __repr__(self):
        return "Ideal on generators %s over %s"%(
            self.__gens, self.__algebra)

    def __call__(self, x):
        raise NotImplementedError
                                
    def __contains__(self, x):
        raise NotImplementedError 
            
    def ambient_algebra(self):
        return self.__ambient_algebra
    
    def base_ring(self):
        return self.__base_ring
    
    def basis(self):
        try:
            return self.__basis
        except:
            basis_helper(self)
        return self.__basis

    def gen(self,i):
        """
        The i-th generator of the algebra ideal.
        """
        n = len(self.__gens)
        if i < 0 or not i < n: 
            raise IndexError, \
                  "Argument i (= %s) must be between 0 and %s."%(i, n-1)
        return self.__gens[i]

    def ngens(self):
        """
        The number of generators of the algebra ideal.
        """
        return len(self.__ngens)
