"""
Quaternion ideal

AUTHOR: David Kohel, 2005-09
"""

#*****************************************************************************
#  Copyright (C) 2005 David Kohel <kohel@maths.usyd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful, 
#    but WITHOUT ANY WARRANTY; without even the implied warranty 
#    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
#  See the GNU General Public License for more details; the full text 
#  is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

__doc_exclude = ['MatrixSpace', 'AlgebraOrderIdeal', 'QuaternionAlgebra_Generic',\
                 'QuaternionOrderElement']

from sage.matrix.matrix_space import MatrixSpace
from algebra_order_ideal import AlgebraOrderIdeal
from quaternion_algebra import QuaternionAlgebra_Generic
from quaternion_order_element import QuaternionOrderElement

def QuaternionOrderIdeal(R, gens):
    if not isinstance(R, Ring):
        raise TypeError, "Argument R (= %s) must be a ring."%R
    if not isinstance(B, (list,tuple)):
        raise TypeError, "Argument B (= %s) must be a list."%B
    if not isinstance(H, QuaternionAlgebra_Generic):
        raise TypeError, \
              "Argument B (= %s) must be a sequence of quaternions."%B
    return QuaternionIdeal_Generic(H,R,B)

class QuaternionOrderIdeal_Generic(AlgebraOrderIdeal):

    def __init__(self, H, R, B):
        """
	"""
        self.__ambient_algebra = H
        self.__basis = B
        self.__base_ring = R
	
    def __call__(self, x):
        if isinstance(x, QuaternionOrderElement) and x.parent() is self: 
	    return x
        return QuaternionOrderElement(self,x)

    def __repr__(self):
        return "Quaternion order over %s with basis %s"%(
            self.__base_ring, self.__basis)

    def gen(self,i):
        """
        The i-th basis element of the quaternion order.
        """
        if i < 0 or not i < 4: 
            raise IndexError, \
                  "Argument i (= %s) must be between 0 and %s."%(i, 3)
        return QuaternionOrderElement(self,self.__basis[i])

    def algebra(self):
        """
	"""
        return self.__algebra
	
    def basis(self):
        """
	"""
        return [ self(x) for x in self.__basis ]
	
    def base_ring(self):
        """
	"""
        self.__base_ring
	
    def __embedding_matrix(self):
        """
	"""
        K = self.__algebra.base_ring()
        M = MatrixSpace(K,4)
        return M([ x.vector() for x in self.__basis ])
	
    def inverse_embedding_matrix(self):
        """
	"""
        try:
            return self.__inverse_embedding_matrix
        except:
            pass
        embedding_matrix = self.__embedding_matrix()
        try: 
            self.__inverse_embedding_matrix = embedding_matrix**-1
            return self.__inverse_embedding_matrix
        except:
            raise AttributeError, \
                  "Basis for quaternion order is not full rank."
	
    def gram_matrix(self):
        R = self.base_ring()
        M = MatrixSpace(R,4,4)(0)
        B = self.basis()
        for i in range(4):
            x = B[i]
            M[i,i] = 2*(x.reduced_norm())
            for j in range(i+1,4):
                y = B[j]
                c = (x * y.conjugate()).reduced_trace()
                M[i,j] = c
                M[j,i] = c
        return M
	
    def random_element(self):
        R = self.base_ring()
        return self([ R.random_element() for _ in range(4) ])
