"""nodoctest
all.py -- much of sage is imported into this module, so you don't
          have to import everything individually. 
"""

#*****************************************************************************
#
#   SAGE: System for Algebra and Geometry Experimentation    
#
#       Copyright (C) 2005 William Stein <wstein@ucsd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************


import os, sys
if sys.version_info[:2] < (2, 4):
    print >>sys.stderr, "Sorry, SAGE requires Python 2.4 or later"
    sys.exit(1)


###################################################################
from sage.libs.pari import *
import sage.libs.ntl as ntl


from sage.interfaces.all import *

from sage.rings.all   import *
from sage.matrix.all  import *
from sage.modules.all import *
from sage.monoids.all import *
from sage.algebras.all import *

from sage.misc.misc import alarm, cputime, verbose, set_verbose, set_verbose_files, get_verbose_files, unset_verbose_files, get_verbose, \
     version, add, union, uniq, powerset, exists, forall, \
     random_sublist, mul, prod

from sage.misc.defaults import set_default_variable_name

from sage.modular.all import *
from sage.schemes.all import *

from sage.libs.mwrank import mwrank_EllipticCurve, mwrank_MordellWeil, set_precision as mwrank_set_precision

#from sage.libs.hanke import hanke_Matrix, hanke_diagonal_form

from sage.modular.dirichlet import *
from sage.modular.congroup import *
from sage.modular.dims import *
from sage.functions.transcendental import *
from sage.groups.all import *
from sage.databases.all import *

from sage.rings.arith import *

from sage.structure.factorization import Factorization

from sage.categories.all import *

###################################################################

from os import system

# maximize memory resources
try:
    import resource   # unix only...
    resource.setrlimit(resource.RLIMIT_AS, (-1,-1))
except:
    pass

# abreviations
EC = EllipticCurve
MS = ModularSymbols
#IM = IntegerModRing
#Integers = IM
#MF = ModularForms
#NF = NumberField
#PR = PolynomialRing
#VS = VectorSpace
true = True
false = False
oo = rings.infinity
x = PolynomialRing(RationalField(),'x').gen(0)

def CS(x):
    return x.cuspidal_subspace()

def NS(x):
    return x.new_subspace()

def OS(x):
    return x.old_subspace()

def ES(x):
    return x.eisenstein_subspace()

def NCMS(n,k=2,sign=1):
    return ms(n,weight=k,sign=sign).cuspidal_subspace().new_subspace()

tm = misc.cputime

QQ = rings.RationalField()
ZZ = rings.IntegerRing()
RR = rings.RealField()  # default real field
CC = rings.ComplexField()
I = CC.gen(0)
i = CC.gen(0)
Q=QQ; Z=ZZ; R=RR; C=CC

from sage.misc.functional import *

from sage.misc.latex import *

##########################################################################
def benchmark(n=-1):
    """
    Run a well-chosen range of SAGE commands and record the time it
    takes for each to run.

    INPUT:
        n -- int (default: -1) the benchmark number; the default
             of -1 runs all the benchmarks.
    OUTPUT:
        list -- summary of timings for each benchmark.
    """
    import sage.misc.benchmark
    return sage.misc.benchmark.benchmark(n)

##########################################################################
# I tried this save/restore session, but it seems hopeless.
# For example, how to restore the local caches of modular symbols
# spaces and vector spaces to the modules?  It's probably better
# to have an object-by-object database anyways.

# def save_session(dirname):
#     """
#     Save the current session to the named directory.  A file is
#     created for each object currently defined.
#     """
#     if os.path.exists(dirname):
#         os.rename(dirname,dirname+".old")
#     os.mkdir(dirname)
#     for x in show_identifiers():
#         db.save(globals()[x], "%s/%s"%(dirname,x))

# import posix
# def load_session(dirname):
#     """
#     Load all variables defined in a session from the named directory.
#     """
#     assert os.path.exists(dirname), "Directory %s does not exist."%dirname
#     for x in posix.listdir(dirname):
#         globals()[x] = db.load("%s/%s"%(dirname,x))

        
#__iglob = globals().keys()[:] + \
#        ['os', '__', '_i1', '_i', '_ii', '___', 'show_identifiers', '_iii', '_', 'iglob']

#def show_identifiers():
#    """
#    Returns a list of the names of non-hidden (i.e., not starting with
#    an underscore) identifiers defined by the user in this session.
#    These are the identifiers saved by the save command.
#    """
#    return [k for k in globals().keys() if not k in __iglob and k[0] != "_"]

###############################################################
# Interactive debugger
import pdb
def debug(code):
    """
    Evaluate code using the interactive debugger and return the
    result.  The string code must be valid python code enclosed in
    quotes; in particular it is not preparsed by SAGE, so use **
    instead of ^ for exponentiation, and explicitly create elements of
    ZZ, QQ, etc.

    INPUT:
        code -- str
        
    REMARKS: This function is extremely powerful!  For example, if you
    want to step through each line of execution of, e.g.,"factor(100)",
    type
    
        sage.: debug("factor(100)")
        
    then at the (Pdb) prompt type "s" (or "step"), then press return
    over and over to step through every line of Python that is called
    in the course of the above computation.   Type ? at any time for
    help on how to use the debugger (e.g., "l" lists 11 lines around
    the current line; "bt" gives a back trace, etc.).

    SETTING A BREAK POINT: If you have some code in a file and would
    like to drop into the debugger at a given point, put the following
    code at that point in the file: \code{import pdb; pdb.set_trace()}
    For an article on how to use the Python debuger, see
       http://www.onlamp.com/pub/a/python/2005/09/01/debugger.html
    """
    return pdb.runeval(code)
    # this could also be useful; it drops
    # us into a debugger in an except block:
    #     import pdb; pdb.post_mortem(sys.exc_info()[2])
        


# Grab signal handler back from PARI
def __my_sigint(x, n):
    raise KeyboardInterrupt

import signal
signal.signal(signal.SIGINT, __my_sigint)
signal.signal(signal.SIGABRT, __my_sigint)

def __my_sigfpe(x, n):
    raise RuntimeError, "An floating point exception occured."
signal.signal(signal.SIGFPE, __my_sigfpe)


# Testing module
import sage.tests.all as tests
