"""
Abstract base class for objects with generators
"""

#*****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@ucsd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.misc.defaults

cdef class Generators:
##     def __getattr__(self, attrname):
##         if attrname[0].lower() in ['g', 'x', '_']:
##             try:
##                 if len(attrname)==1:
##                     n = 0
##                 else:
##                     n=int(attrname[1:])
##             except:
##                 raise AttributeError, attrname
##             if n<0 or n>=self.ngens():
##                 raise AttributeError, "Generator (=%s) must be between 0 and %s"%\
##                       (n,self.ngens()-1)
##             return self.gen(n)
##         else:


##             ##############################################################
##             # You attempted to access an undefined function or variable. #
##             raise AttributeError, "The attribute '%s' is not defined"%attrname
##             ##############################################################


        
            
    def __getitem__(self, n):
        return self.list()[n]

    def __getslice__(self, n, m):
        return self.list()[n:m]
    
    def list(self):
        """
        Return a list of all elements in this object, if possible.
        """
        from sage.rings.infinity import infinity
        try:
            return self.__list
        except AttributeError:
            pass
        v = []
        G = self.gens()
        if len(G) == 0:
            self.__list = v
            return
        cnt = [0] * len(G)
        try:
            stop = []
            for g in G:
                stop.append(g.order()-1)
            for x in stop:
                if x == infinity:
                    raise ArithmeticError, "%s is not finite."%self
        except AttributeError:
            raise ArithmeticError, "Unable to compute list of all elements of %s"%self
        n = 0
        z = self(1)
        v.append(z)
        while cnt != stop:
            z = z * G[0]
            cnt[0] = cnt[0] + 1
            i = 0
            while i < len(cnt)-1 and cnt[i] > stop[i]:
                cnt[i] = 0
                cnt[i+1] = cnt[i+1] + 1
                z = z * G[i+1]
                i = i + 1
            #end
            v.append(z)
        #end
        self.__list = v
        return self.__list
        
    def gens(self):
        """
        Return a tuple whose entries are the generators for this
        object, in order.
        """
        cdef int i, n
        try:
            return self.__gens
        except AttributeError:
            v = []
            n = self.ngens()
            for i from 0 <= i < n:
                v.append(self.gen(i))
            self.__gens = tuple(v)
            return self.__gens
            

    def ngens(self):
        return 0
    
    def assign_names(self, names=None):
        if names is None: return
        if isinstance(names, str):
            names = sage.misc.defaults.variable_names(self.ngens(), names)
        else:
            if not isinstance(names, (list, tuple)):
                raise TypeError, "names must be a list or tuple of strings"
            for x in names:
                if not isinstance(x,str):
                    raise TypeError, "names must consist of strings"
            if len(names) != self.ngens():
                raise IndexError, "the number of names must equal the number of generators"
        self.__names = tuple(names)
        try:
            del self.__name
        except AttributeError: pass

    def names(self):
        try:
            return self.__names
        except AttributeError:
            self.__names = sage.misc.defaults.variable_names(self.ngens())
            return self.__names
        
    def name(self):
        try:
            return self.__name
        except AttributeError:
            v = self.names()
            if len(v) != 1:
                raise TypeError, "there must be exactly one generator"
            self.__name = v[0]
            return self.__name
            
