"""
Interface to the gp interpreter
"""

#*****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@ucsd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import os
import pexpect

from sage.misc.misc import SAGE_HOME

class Expect:
    def __init__(self, name, prompt, command=None, maxread=10000, 
                 script_subdirectory="", restart_on_ctrlc=False):
        if command == None:
            command = name
        self.__maxread = maxread
        self.__script_subdirectory = script_subdirectory
        self.__name = name
        self.__command = command
        self.__prompt = prompt
        self.__restart_on_ctrlc = restart_on_ctrlc
        self._start()

    def expect(self):
        return self.__expect

    def _change_prompt(self, prompt):
        self.__prompt = prompt

    def _start(self):
        current_path = os.path.abspath('.')
        dir = '%s/data/scripts/%s/%s'%(SAGE_HOME,self.__name, self.__script_subdirectory)
        if not os.path.exists(dir):
            os.makedirs(dir)
        os.chdir(dir)
        self.__expect = pexpect.spawn(self.__command)
        os.chdir(current_path)
        self.__expect.timeout = 999999999
        self.__expect.expect(self.__prompt)
        self.__expect.setmaxread(self.__maxread)

    def __del__(self):
        # The following two lines program around a bug in pexpect.
        def dummy(): pass   
        self.__expect.close = dummy
    
    def __call__(self, line):
        if line.find('\n') != -1:
            raise ValueError, "line must not contain any newlines"
        try:
            self.__expect.sendline(line)
            self.__expect.expect(self.__prompt)
            out = self.__expect.before
        except KeyboardInterrupt:
            if self.__restart_on_ctrlc:
                self.__expect.close()
                self._start()
                raise KeyboardInterrupt, "Restarting %s"%self.__name
            else:
                self.__expect.sendline(chr(3))  # send ctrl-c
                self.__expect.expect(self.__prompt)
                self.__expect.expect(self.__prompt)
                raise KeyboardInterrupt, "Ctrl-c pressed"

        i = out.find("\n")
        j = out.rfind("\r")
        return out[i+1:j].replace('\r\n','\n')

    def eval(self, code):
        return '\n'.join([self(L) for L in code.split('\n')])



